<?php

/**
 * @file
 * DNS provisioning module.
 *
 * The goal of this module is to manage DNS zonefiles and Resource Records
 * (RRs), for sites that are about to be created.  It uses the provision API to
 * tie into the right places in the site creation work flow.
 */

/**
 * Implements hook_drush_init().
 */
function hosting_dns_drush_init() {
  hosting_dns_provision_register_autoload();
}

/**
 * Register our directory as a place to find provision classes.
 */
function hosting_dns_provision_register_autoload() {
  static $loaded = FALSE;
  if (!$loaded) {
    $loaded = TRUE;
    provision_autoload_register_prefix('Provision_', dirname(__FILE__));
  }
}

/**
 * Implements hook_drush_command().
 */
function hosting_dns_drush_command() {
  $items['provision-dns-obtain'] = array(
    'description' => dt('Fetches DNS entries from hostmaster'),
    'bootstrap' => DRUSH_BOOTSTRAP_DRUPAL_LOGIN,
    'hidden' => TRUE,
  );

  $items['provision-dns-deploy'] = array(
    'description' => dt('Deploy a zonefile, based on all DNS nodes associated with a given zone.'),
    'bootstrap' => DRUSH_BOOTSTRAP_DRUPAL_LOGIN,
    'examples' => array(
      'drush @site provision-dns-deploy' => dt('Deploys a zonefile for @site.'),
    ),
  );

  $items['provision-dns-zone-convert'] = array(
    'description' => dt('Converts a possibly existing php zonefile into DNS nodes. This should only be ran when enabling this module for the first time.'),
    'bootstrap' => DRUSH_BOOTSTRAP_DRUPAL_LOGIN,
    'options' => array(
      'sanity_check' => dt('Sanity check'),
    ),
    'hidden' => TRUE,
  );

  return $items;
}

/**
 * Implements drush_hook_COMMAND_validate().
 *
 * Make sure this command runs on the hostmaster site.
 */
function drush_hosting_dns_provision_dns_obtain_validate() {
  // This function can only be ran on hostmaster.
  if (d()->name != '@hostmaster') {
    drush_set_error('This command can only be ran on @hostmaster.');
  }
}

/**
 * Implements drush_hook_COMMAND().
 *
 * Function for obtaining.
 */
function drush_hosting_dns_provision_dns_obtain($uri) {
  drush_log('Obtaining site (' . $uri . ') from hostmaster.');

  $site = hosting_get_site_by_url($uri, FALSE);

  $query = new EntityFieldQuery();
  $result = $query->entityCondition('entity_type', 'node')
    ->entityCondition('bundle', 'dns')
    ->propertyCondition('status', NODE_PUBLISHED)
    ->entityCondition('site', $site)
    ->execute();

  if (isset($result['node'])) {
    $dns_entry_keys = array_keys($result['node']);
    $dns_entries = entity_load('node', $dns_entry_keys);
    return $dns_entries;
  }

  return array();
}

/**
 * Implements drush_hook_COMMAND_validate().
 *
 * Make sure the site is installed and enabled.
 */
function drush_hosting_dns_provision_dns_deploy_validate() {
  if (!@drush_bootstrap(DRUSH_BOOTSTRAP_DRUPAL_CONFIGURATION)) {
    if (drush_get_option('force', FALSE)) {
      drush_log("clearing error");
      drush_set_context('DRUSH_ERROR_CODE', DRUSH_SUCCESS);
    }
  }
  if (!drush_get_option('installed') && !drush_get_option('force', FALSE)) {
    drush_set_error('PROVISION_DRUPAL_SITE_NOT_FOUND');
  }
}

/**
 * Implements drush_hook_COMMAND().
 *
 * Function for generating a zonefile based on DNS nodes.
 *
 * @return bool
 */
function drush_hosting_dns_provision_dns_deploy() {
  $zone = d()->uri;

  // Obtain the zone.
  drush_log('Attempting to obtain DNS entries for this site.');
  $dns_entries = provision_backend_invoke('@hostmaster', 'provision-dns-obtain', array($zone));

  drush_log('Found ' . count($dns_entries['object']) . ' DNS entries.', 'success');

  // Parse the zonefile.
  foreach ($dns_entries['object'] as $entry) {

    drush_print_r($entry);

    $name = $entry['title'];
    $type = $entry['dns_type'];
    $destination = $entry['dns_value'];

    // Todo: This still uses the ZONE.zone.inc file, which needs to be removed.
    $record = d()->service('dns')->config('zone', $zone)->record_get($name);
    if (!is_array($record[$type])) {
      $record[$type] = array();
    }
    $record[$type] = array_merge($record[$type], array($destination));
    $status = d()->service('dns')->config('zone', $zone)->record_set($name, $record)->write();

    drush_log("Wrote $type-record '$name' with destination '$destination'.", 'success');
  }

  // Deploy the zonefile.
  d()->service('dns')->parse_configs();

  // Todo: Make sure to pass all statuses not only the last one.
  return $status;
}


/**
 * Implements drush_hook_COMMAND().
 *
 * Function for converting existing zonefiles into DNS nodes.
 *
 * @return bool
 */
function drush_hosting_dns_provision_dns_zone_convert() {
  $sanity_check = drush_get_option('sanity_check');

  if ($sanity_check !== TRUE) {
    $status = drush_set_error('DRUSH_WRONG_ARGUMENT', dt("Sanity check failed. You really should not be running this manually."));
  }

  // Todo: See if an aegir_root / config / dns.d / ZONE.zone.inc file exists

  // Todo: Parse the ZONE.zone.inc file.

  // Todo: Save corresponding DNS nodes on the zone.

  // Todo: Remove the file, don't use it anymore.

  return $status;
}

// Todo: Clean this up. For now it is a nice reference :)
//
//function drush_hosting_dns_zone($action, $zone, $name = null, $type = null, $destination = null) {
//  switch ($action) {
//  case 'create-host':
//    $status = d()->service('dns')->create_host($zone);
//    break;
//  case 'delete-host':
//    $status = d()->service('dns')->delete_host($zone);
//    break;
//  case 'verify':
//  case 'create':
//    $status = d()->service('dns')->create_zone($zone);
//    break;
//  case 'delete':
//    $status = d()->service('dns')->delete_zone($zone);
//    break;
//  case 'rr-add':
//    $record = d()->service('dns')->config('zone', $zone)->record_get($name);
//    if (!is_array($record[$type])) {
//      $record[$type] = array();
//    }
//    $record[$type] = array_merge($record[$type], array($destination));
//    $status = d()->service('dns')->config('zone', $zone)->record_set($name, $record)->write();
//    break;
//  case 'rr-modify':
//    $status = d()->service('dns')->config('zone', $zone)->record_set($name, array($type => array($destination)))->write();
//    break;
//  case 'rr-delete':
//    if ($type) {
//      $status = d()->service('dns')->config('zone', $zone)->record_set($name, array($type => NULL))->write();
//    } else {
//      $status = d()->service('dns')->config('zone', $zone)->record_set($name, NULL)->write();
//    }
//    break;
//  default:
//    $status = drush_set_error('DRUSH_WRONG_ARGUMENT', dt("wrong argument provided to hosting-dns-zone"));
//    break;
//  }
//  // Todo: we shouldn't reload zones unless we have actually done changes, but return values of the above functions are always FALSE
//  d()->service('dns')->parse_configs();
//
//  return $status;
//}

/**
 *  Implements hook_provision_services().
 */
function hosting_dns_provision_services() {
  hosting_dns_provision_register_autoload();
  return array('dns' => NULL);
}